#include "plotxy.h"

void LinScale::make(float a,float e,float A,float E) {
  e-=a; if (!e) return;	// illegal
  E-=A; if (!E) return;	// illegal
  f=E/e;
  O=A-a*f;
}

const COLORREF colors[]={RGB(255,0,0),RGB(255,255,0),RGB(128,128,255),RGB(64,128,64)};

void Trace::deallocate() {
  if (ptLine) {delete[] ptLine; ptLine=0;}
  if (ptMinMax) {delete[] ptMinMax; ptMinMax=0;}
  if (ptStdDev) {delete[] ptStdDev; ptStdDev=0;}
}

void Trace::allocGdi(int index) {
  COLORREF cr=colors[index];
  Gdiplus::Color c(GetRValue(cr),GetGValue(cr),GetBValue(cr));
  pnLine=new Gdiplus::Pen(c);
  brLine=new Gdiplus::SolidBrush(c);
  c.SetValue(c.GetValue()&0xC0FFFFFF);
  brStdDev=new Gdiplus::SolidBrush(c);
  c.SetValue(c.GetValue()&0x80FFFFFF);
  brMinMax=new Gdiplus::SolidBrush(c);
// Erst mal kein Polygon
}

void Trace::deallocGdi() {
  if (brPoly) {delete brPoly; brPoly=0;}
  if (brMinMax) {delete brMinMax; brMinMax=0;}
  if (brStdDev) {delete brStdDev; brStdDev=0;}
  if (brLine) {delete brLine; brLine=0;}
  if (pnLine) {delete pnLine; pnLine=0;}
}

void Trace::make() {
  if (polylen!=ydata->size()) deallocate();
  polylen=ydata->size();
  if (!nLines) nLines=1;
  if (!ptLine) ptLine=new Gdiplus::PointF[polylen*nLines];
  if (ydata->mux()>1 && !ptMinMax) ptMinMax=new Gdiplus::PointF[polylen<<1];
  if (ydata->mux()>1 && !ptStdDev) ptStdDev=new Gdiplus::PointF[polylen<<1];
  
  for (int i=0; i<polylen; i++) {
    Datapoint dp;
    ydata->get(i,dp);
    float X=xaxis->scaleInf(i);
    ptLine[i].X=X;
    ptLine[i].Y=yaxis->scaleInf(dp.mean);
    if (ydata->mux()>1) {
      int j=(polylen<<1)-1-i;
      ptMinMax[j].X=ptMinMax[i].X=X;
      ptMinMax[i].Y=yaxis->scaleInf(dp.max);
      ptMinMax[j].Y=yaxis->scaleInf(dp.min);
      ptStdDev[j].X=ptMinMax[i].X=X;
      ptStdDev[i].Y=yaxis->scaleInf(dp.mean+dp.var);
      ptStdDev[j].Y=yaxis->scaleInf(dp.mean-dp.var);
    }
  }
}

void Trace::paintPoly(Gdiplus::Graphics&g) const{
  if (brPoly && ptPoly) g.FillPolygon(brPoly,ptPoly,polylen<<1);
}

void Trace::paintMinMax(Gdiplus::Graphics&g) const{
  if (brMinMax && ptMinMax) g.FillPolygon(brMinMax,ptMinMax,polylen<<1);
}

void Trace::paintStdDev(Gdiplus::Graphics&g) const{
  if (brStdDev && ptStdDev) g.FillPolygon(brStdDev,ptStdDev,polylen<<1);
}

void Trace::paintLine(Gdiplus::Graphics&g) const{
  g.DrawLines(pnLine,ptLine,polylen);
}

extern HINSTANCE hInstance;


static struct PlotXY:public WNDCLASS{
 PlotXY();
 static LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);
}plotxy;

LRESULT CALLBACK PlotXY::WndProc(HWND,UINT,WPARAM,LPARAM) {
 return 0;
}

PlotXY::PlotXY() {
 style=CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS|CS_PARENTDC;
 lpfnWndProc=WndProc;
 cbWndExtra=sizeof this;
 hInstance=::hInstance;
 RegisterClass(this);
}
