#pragma once
#include <windows.h>
#include <windowsx.h>
#include <uxtheme.h>
#include <limits>
#include <gdiplus.h>

template<class T> void setMax(T&a,T b) {if (!(a>=b)) a=b;}
template<class T> void setMin(T&a,T b) {if (!(a<=b)) a=b;}

const int FINFI=0x7F800000;
#define FINF *(float*)&FINFI

#undef max
#undef min

template<class T> struct Range{
  T a,e;
  T delta() const {return e-a;}
  void reset() {init(std::numeric_limits<T>::max(),std::numeric_limits<T>::min());}
  void init(T b, T c) {a=b; e=c;}
  void init() {init(0,0);}
  bool expand(T b) {return expand(b,b);}
  bool expand(T b, T c) {bool r=false;if(!(a<=b)){a=b;r=true;}if(!(e>=c)){e=c;r=true;}return r;}
};

class Scale{
public:
  virtual void make(float,float,float,float)=0;	// Zweipunktkalibrierung
  virtual float scale(float) const=0;
  virtual float unscale(float) const=0;
};

class LinScale:public Scale{
  float f,O;
  virtual void make(float a,float e,float A,float E);
  virtual float scale(float x) const {return x*f+O;}
  virtual float unscale(float X) const {return (X-O)/f;}
public:
  LinScale(float s=1,float o=0):f(s),O(o) {}
};

struct Axis{
 Range<float>r,R;
 Scale*scale;
 float scaleInf(float x) {
  /*if (isfinite(x))*/ return scale->scale(x);
 }
 void make() {scale->make(r.a,r.e,R.a,R.e);}
};

extern const COLORREF colors[];

struct Data{
  virtual float get(int) const=0;
};

union Datapoint{
  float a[4];
  struct{
    float mean,min,max,var;	// Mittelwert, Minimum, Maximum, Varianz
  };
};

struct Tracedata{
  virtual size_t size() const=0;// Polygonlnge, sollte 1024 nicht berschreiten (TODO: Fr zu betrachtenden Ausschnitt!!)
  virtual size_t mux() const=0;	// bereinander gezeichnete Linien
  virtual void get(int,Datapoint&) const=0;
};

struct Trace{
  Gdiplus::Pen   *pnLine;
  Gdiplus::Brush *brLine,*brMinMax,*brStdDev,*brPoly;
  Gdiplus::PointF*ptLine,*ptMinMax,*ptStdDev,*ptPoly;
  void deallocate();
  void allocGdi(int index);
  void deallocGdi();
  int polylen;
  int nLines;	// bei Mehrfachtrigger und <=4 Ergebnis-Linien, sonst 1
  bool lineAsPoly;
  Axis*xaxis,*yaxis;
  Tracedata*ydata;
  void make();	// Generiere Punktkoordinaten fr Hauptfenster, noch nicht fr Miniatur
  void paintPoly(Gdiplus::Graphics&)const;	// Polygon zwischen Kurvenzgen
  void paintMinMax(Gdiplus::Graphics&)const;	// Unterste Polygonlage
  void paintStdDev(Gdiplus::Graphics&)const;	// Darberliegende Polygonlage
  void paintLine(Gdiplus::Graphics&)const;	// Kurvenzug
  void paintMarkers(Gdiplus::Graphics&)const;	// Punkt-Marker
};
